/*-------------------------------------------------------------------------
*
* Copyright (c) 2004-2008, PostgreSQL Global Development Group
*
* IDENTIFICATION
*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Statement.java,v 1.115 2009/09/26 15:21:21 jurka Exp $
*
*-------------------------------------------------------------------------
*/

/*
 * $URL$
 * $Id$
 * $Revision$
 */
package org.postgresql.core;

import java.sql.SQLException;
import java.util.TimerTask;

/**<b>Important thread-safty notice!</b><br/>
 * If you add this task to timer queue ensure, that
 * two different task with same connection will not
 * cause to accidently interrupt other queries.
 * Query cancelation cancels only <b>currently</b> executed
 * query, not the given query.
 *
 * @author Radosław Smogura(rsmogura@softperience.pl)
 * @version (Nadia)
 */
public class QueryTimeoutTimerTask extends TimerTask {
    /** Holds additional monitor. */
    private Object monitor;
    
    /** Holds connection used to cancel query. */
    private BaseConnection connection;

    /** Currentlu not used, but in futere can be used for logging etc... */
    private BaseStatement statement;

    /** Holds value indicating if query has ended. */
    private volatile boolean taskEnded;

    /** Creates the new timer task, when this task will be added to queu end
     *  executed the query (if not ended will terminate)
     *
     * @param monitor the additinal monitor to use for syncing between query result (needed), query cancelation
     * @param connection the connection used to send cancelation request
     * @param statement the executing statement to wath it's timeout
     */
    public QueryTimeoutTimerTask(Object monitor, BaseConnection connection, BaseStatement statement) {
        super();
        this.monitor = monitor;
        this.connection = connection;
        this.statement = statement;
    }
    @Override
    public void run() {
        synchronized (getMonitor()) {
            if (!taskEnded) {
                try {
                    if (!connection.isClosed())
                        connection.cancelQuery();
                }catch(SQLException ignored) {
                    //TODO Should we add or log error or silently ignore it?
                }finally {
                    //And task if we got eny error - to prevent dead locks
                    synchronized(this) {
                        taskEnded = true;
                        this.notifyAll();
                    }
                }
            }
        }
    }

    /** Notifies that this task is unneeded, query has ended.
     * 
     *  Before calls {@link TimerTask#cancel()} it sets flag indicating
     *  that query has ended.
     *
     * @return the {@link TimerTask#cancel()}
     */
    @Override
    public boolean cancel() {
        synchronized(getMonitor()) {
            this.taskEnded = true;
        }
        return super.cancel();
    }

    /** Wait till this task will end, if not ended. */
    public void join() throws InterruptedException{
        if (!taskEnded) {
            synchronized(this) {
                if (!taskEnded) {
                    this.wait();
                }
            }
        }
    }

    public Object getMonitor() {
        return monitor;
    }

    public void setMonitor(Object monitor) {
        this.monitor = monitor;
    }

    public BaseConnection getConnection() {
        return connection;
    }

    public void setConnection(BaseConnection connection) {
        this.connection = connection;
    }

    public BaseStatement getStatement() {
        return statement;
    }

    public void setStatement(BaseStatement statement) {
        this.statement = statement;
    }
}
